/**
*
*    thousand
*
*    A program which generates new words by choosing a word
*    from each of two large word lists (each in a different
*    text file).
*
*    Jeff Rivett
*    August 22, 1989
*
*    NOTE: requires COMPACT memory model for compilation.
**/

#include "stdio.h"
#include "stdlib.h"
#include "time.h"
#include "dos.h"
#include "conio.h"
#include "string.h"
#include "ctype.h"
#include "alloc.h"

#define AUTHOR "Jeff Rivett"
/* #define VERSION "1.21"
#define VERSION_DATE "August 10, 1990" */
#define VERSION "1.22"
#define VERSION_DATE "June 17, 1995"

#define TRUE  1
#define FALSE 0

#define PREFIX  "KDEFAULT.PRE"
#define SUFFIX  "KDEFAULT.SUF"

#define MAX 2000

#define NO_DEF		"???"

#define FORMFEED     '\014'

#define PR_P   0
#define PR_S   1
#define PR_MAX    0
#define PR_PTR    1
#define PR_CHR   2

void make_filename(char *name_str);
void display_word(char *p,char *s,char *pd,char *sd,char prior,int fast);
void show_help(void);

char (huge *prefixes)[20];
char (huge *pref_def)[30];
char (huge *suffixes)[20];
char (huge *suff_def)[30];

/**
*
*        Functions
*
**/

void show_help()
{
	int ch;

	clrscr();
	highvideo();
	cprintf("Extended help for Thousand\r\n");
	lowvideo();
	cprintf("\r\n");
	cprintf("   The program is always in either Prefix or Suffix priority mode.\r\n");
	cprintf("   All messages that refer to \'group\' refer to all listable words\r\n");
	cprintf("   with the current prefix or suffix (whichever has priority).\r\n");
	cprintf("\r\n");
	cprintf("   Priority is indicated at the beginning of the word line.\r\n");
	cprintf("   P indicates Prefix priority, while S means Suffix priority.\r\n");
	cprintf("\r\n");
	cprintf("   Priority controls the operation of most of the commands. For\r\n");
	cprintf("   example, the up and down arrow keys decrement and increment\r\n");
	cprintf("   (respectively) either the prefix or the suffix, whichever does\r\n");
	cprintf("   NOT have priority.\r\n");
	cprintf("\r\n");
	cprintf("   Printing and file commands are similarly dependant on priority.\r\n");
	cprintf("   For example, if the file mode is ->group, the priority is Prefix,\r\n");
	cprintf("   and you press F3, all the words with the current prefix from the\r\n");
	cprintf("   first suffix to the current suffix will be written to the open file.\r\n");
	cprintf("\r\n");
	cprintf("   File and print modes that have arrows in them are of two types.\r\n");
	cprintf("   An arrow in front indicates that words from the beginning of the\r\n");
	cprintf("   lists or group up to the current group or word will be included.\r\n");
	cprintf("   An arrow after the mode indicates \'current\' to \'end\'.\r\n");
	cprintf("\r\n");
	cprintf("Press any key...");
	ch = getch();
	if (ch == '\0') getch();

    return;
}

void display_word(p,s,pd,sd,prior,fast)

	char *p,*s,*pd,*sd;
	char prior;
	int fast;
	{
		gotoxy(1,24);
		clreol();
		gotoxy(1,23);
		clreol();
		cprintf("\r   %s%s",p,s);
		if (!fast)
		{
			cprintf("   ");
			cprintf("%s\n",pd);
			highvideo();
			gotoxy(strlen(p)+strlen(s)+4,24);
			cprintf(" + ");
			lowvideo();
			cprintf("%s",sd);
		}
		highvideo();
        gotoxy(1,23);
		cprintf("\r%c> ",prior);
		lowvideo();
        return;
    }

void make_filename(name_str)

	char *name_str;
	{
		char newname[30];
		int len,idx;
		int new_idx = 0;
		char ch;
		int name_len = 0;
		int ext_len = 0;
		int dot_found = FALSE;

		len = strlen(name_str);
		for (idx=0;idx<len;idx++)
		{
			ch = toupper(name_str[idx]);
			switch (ch)
			{
				case ('.'):
				{
					if (!dot_found)
					{
						dot_found = TRUE;
						newname[new_idx] = '.';
						new_idx++;
					}
					break;
				}
				default:
				{
					if (!dot_found)
					{
						if (name_len < 8)
						{
							newname[new_idx] = ch;
							new_idx++;
							name_len++;
						}
					}
					else
					{
						if (ext_len < 3)
						{
							newname[new_idx] = ch;
							new_idx++;
							ext_len++;
						}
					}
					break;
				}
			}
		}
		newname[new_idx] = '\0';
		strcpy(name_str,newname);
		return;
	}

/*******
*
*        Main Program
*
**/

main(argc,argv)

    int argc;
    char *argv[];
    {
        FILE *prefix_file;
        FILE *suffix_file;
        FILE *out_file;
        FILE *prnt_dev;

		char pfile_name[30],sfile_name[30];
		char buffer[30];
		char *ofile_name;

		char ch;
		char c;
		int ci;
/*		char word[50];  */
		char in_line[80];
		int delim;
        int done = FALSE;
		int sound_flag = TRUE;
		int p_indx, s_indx;
        int p_max, s_max;
        int priority, non_prior;
		long pr_arr[2][5];       /* actually backwards */
/*		int r_ptr;   */
        int sort_flag;
        int help_flag;
        int cancel;
        int file_flag;
        char temp[50];
        int rand_flag;
		char command;
		char fcmd = '1';
		char pcmd = '1';
        int show_word;
		int got_pcmd,got_fcmd;
        int i,j;

/*		char prefixes[MAX][20];
		char pref_def[MAX][30];
		char suffixes[MAX][20];
		char suff_def[MAX][30];
*/
		prefixes = farcalloc(sizeof(*prefixes), MAX);
		pref_def = farcalloc(sizeof(*pref_def), MAX);
		suffixes = farcalloc(sizeof(*suffixes), MAX);
		suff_def = farcalloc(sizeof(*suff_def), MAX);
		if ((prefixes==NULL) || (pref_def==NULL) || (suffixes==NULL) || (suff_def==NULL))
		{
			printf("Not enough memory for prefix/suffix lists.\n\n");
			exit(0);
		}

        prnt_dev = fopen("LPT1","w");

		strcpy(pfile_name,PREFIX);
		strcpy(sfile_name,SUFFIX);
		if (argc > 1)
		{
			for (i=2; i<=argc; i++)
			{
				strcpy(temp,*++argv);
				if (temp[0] == '/')
				{
					if (temp[1]=='?')
					{
						printf("\n\n");
						printf("Command line options for THOUSAND:\n\n\n");
						printf("  /p:[prefix file name]  (default: KDEFAULT.PRE)\n\n");
						printf("  /s:[suffix file name]  (default: KDEFAULT.SUF)\n\n");
						printf("  /?  -  displays this help screen\n\n");
						exit(0);
					}
					if (toupper(temp[1]) == 'P')
					{
						strcpy(pfile_name,temp+3);
					}
					if (toupper(temp[1]) == 'S')
					{
						strcpy(sfile_name,temp+3);
					}
				}
			}
		}
		make_filename(pfile_name);
		make_filename(sfile_name);

		ofile_name = buffer;

		prefix_file = fopen(pfile_name,"r");
        if (prefix_file == NULL)
        {
            printf("Couldn't open PREFIX file (%s).\n",pfile_name);
            exit(2);
        }
		suffix_file = fopen(sfile_name,"r");
        if (suffix_file == NULL)
        {
            printf("Couldn't open SUFFIX file (%s).\n",sfile_name);
            exit(3);
        }

		/* random stuff */
		randomize();

        printf("THOUSAND initialization...");

        printf("\n\nReading prefixes...");
        p_indx = 0;
        do
		{
			fgets(in_line,80,prefix_file);
			ci = 0;
			c = in_line[ci];
			while ((c != '\0') && (c != '\n') && (c != ';'))
			{
				 prefixes[p_indx][ci] = c;
				 c = in_line[++ci];
			}
			prefixes[p_indx][ci] = '\0';
			delim = ci;
			if (c != ';')
			{
				pref_def[p_indx][0] = '\0';
				strcpy(pref_def[p_indx],NO_DEF);
			}
			else
			{
				delim++;
				ci = 0;
				c = in_line[ci+delim];
				while ((c != '\0') && (c != '\n'))
				{
					pref_def[p_indx][ci] = c;
					c = in_line[(++ci)+delim];
				}
				pref_def[p_indx][ci] = '\0';
			}
			p_indx++;
        }
		while (!feof(prefix_file) && !(p_indx > MAX));
        p_max = p_indx - 2;

        printf("\nReading suffixes...");
        s_indx = 0;
        do
        {
			fgets(in_line,80,suffix_file);
			ci = 0;
			c = in_line[ci];
			while ((c != '\0') && (c != '\n') && (c != ';'))
			{
				 suffixes[s_indx][ci] = c;
				 c = in_line[++ci];
			}
			suffixes[s_indx][ci] = '\0';
			delim = ci;
			if (c != ';')
			{
				suff_def[s_indx][0] = '\0';
				strcpy(suff_def[s_indx],NO_DEF);
			}
			else
			{
				delim++;
				ci = 0;
				c = in_line[ci+delim];
				while ((c != '\0') && (c != '\n'))
				{
					suff_def[s_indx][ci] = c;
					c = in_line[(++ci)+delim];
				}
				suff_def[s_indx][ci] = '\0';
			}
			s_indx++;
        }
		while (!feof(suffix_file) && !(s_indx > MAX));
        s_max = s_indx - 2;

/*
		printf("\n\nSorting prefixes...");
        sort_flag = FALSE;
        while (!sort_flag)
        {
            sort_flag = TRUE;
            for (p_indx=0; p_indx<p_max; p_indx++)
            {
                if (strcmp(prefixes[p_indx],prefixes[p_indx+1]) > 0)
                {
                    sort_flag = FALSE;
                    strcpy(temp,prefixes[p_indx]);
                    strcpy(prefixes[p_indx],prefixes[p_indx+1]);
                    strcpy(prefixes[p_indx+1],temp);
					strcpy(temp,pref_def[p_indx]);
					strcpy(pref_def[p_indx],pref_def[p_indx+1]);
					strcpy(pref_def[p_indx+1],temp);
#ifdef DEBUG
                    printf("%s, %s\n",prefixes[p_indx],prefixes[p_indx+1]);
#endif
                }
            }
        }

        printf("\nSorting suffixes...");
        sort_flag = FALSE;
        while (!sort_flag)
        {
            sort_flag = TRUE;
            for (s_indx=0; s_indx<s_max; s_indx++)
            {
                if (strcmp(suffixes[s_indx],suffixes[s_indx+1]) > 0)
                {
                    sort_flag = FALSE;
                    strcpy(temp,suffixes[s_indx]);
                    strcpy(suffixes[s_indx],suffixes[s_indx+1]);
                    strcpy(suffixes[s_indx+1],temp);
					strcpy(temp,suff_def[s_indx]);
					strcpy(suff_def[s_indx],suff_def[s_indx+1]);
					strcpy(suff_def[s_indx+1],temp);
#ifdef DEBUG
                    printf("%s, %s\n",suffixes[s_indx],suffixes[s_indx+1]);
#endif
                }
            }
		}
*/
		delay(2000);

        pr_arr[PR_P][PR_MAX] = p_max;
        pr_arr[PR_S][PR_MAX] = s_max;
        pr_arr[PR_P][PR_PTR] = 0;
        pr_arr[PR_S][PR_PTR] = 0;
        pr_arr[PR_P][PR_CHR] = 'P';
        pr_arr[PR_S][PR_CHR] = 'S';

        priority = PR_P;
        help_flag = TRUE;
        show_word = TRUE;

        file_flag = FALSE;

        while (!done)
        {
            non_prior = ((priority==PR_S)?PR_P:PR_S);
#ifdef DEBUG
            printf("priority: %d, non_prior: %d\n",priority,non_prior);
#endif
            if (help_flag)
            {
				clrscr();
				highvideo();
				cprintf("Thousand-sided dice v%s - by %s - %s\r\n\r\n",VERSION,AUTHOR,VERSION_DATE);
				lowvideo();
				cprintf("A word generator. Press F1 for detailed help.\r\n\r\n");
				highvideo();
				cprintf("Command menu");
				lowvideo();
				cprintf("     (%d Prefixes, %d Suffixes)\r\n\r\n",p_max+1,s_max+1);
				cprintf("LEFT/RIGHT ARROWS: change Prefix/Suffix group priority\r\n");
				cprintf("   UP/DOWN ARROWS: previous/next word in (prefix/suffix) group\r\n");
				cprintf("         HOME/END: top/bottom of (prefix/suffix) group\r\n");
				cprintf("        PGUP/PGDN: next (prefix/suffix) group\r\n");
				cprintf("   CTRL-PGUP/PGDN: first/last (prefix/suffix) group\r\n");
				cprintf("    CTRL-HOME/END: top/bottom of both lists\r\n\r\n");
				cprintf("        F1: detailed help\r\n");
				cprintf("        F2: send words to system printer   (Mode: ");
				switch (pcmd)
				{
					case '1':
						cprintf("word");
						break;
					case '2':
						cprintf("->group");
						break;
					case '3':
						cprintf("group->");
						break;
					case '4':
						cprintf("group");
						break;
					case '5':
						cprintf("->all");
						break;
					case '6':
						cprintf("all->");
						break;
					case '7':
						cprintf("all");
				}
				cprintf(")\r\n");
				cprintf("        F3: send words to open file        (");
				if (file_flag)
				{
					cprintf("%s, Mode: ",ofile_name);
					switch (fcmd)
					{
						case '1':
							cprintf("word");
							break;
						case '2':
							cprintf("->group");
							break;
						case '3':
							cprintf("group->");
							break;
						case '4':
							cprintf("group");
							break;
						case '5':
							cprintf("->all");
							break;
						case '6':
							cprintf("all->");
							break;
						case '7':
							cprintf("all");
					}
					cprintf(")\r\n");
				}
				else cprintf("no open file)\r\n");
				cprintf("\r\n");
				highvideo();
				cprintf("Also: ");
				lowvideo();
				cprintf("F)ile open & setup            S)ound on/off toggle (now ");
				if (sound_flag)
					cprintf("ON)\r\n");
				  else cprintf("OFF)\r\n");
				cprintf("      C)lose output file            ");
				highvideo();
				cprintf("R");
				lowvideo();
				cprintf(")oll dice for new word\r\n");
				cprintf("      P)rinter setup                  (any key to stop)\r\n");
				cprintf("      E)ject paper from printer     Q)uit the program\r\n");
				printf("\n");
                help_flag = FALSE;
            }

            if (show_word)
            {
#ifdef DEBUG
				printf("%d>%d>",pr_arr[PR_P][PR_PTR],pr_arr[PR_S][PR_PTR]);
#endif
                show_word = FALSE;
				display_word(prefixes[pr_arr[PR_P][PR_PTR]],suffixes[pr_arr[PR_S][PR_PTR]],pref_def[pr_arr[PR_P][PR_PTR]],suff_def[pr_arr[PR_S][PR_PTR]],pr_arr[priority][PR_CHR],FALSE);
            }

			/* get command key */
            ch = getch();
            switch (toupper(ch))
			{
				case ('S'):
				{
					sound_flag = (sound_flag?FALSE:TRUE);
					help_flag = TRUE;
					show_word = TRUE;
					break;
				}
                case ('F'):
                {
					cancel = FALSE;
					show_word = TRUE;
                    while ((!file_flag) && (!cancel))
                    {
						gotoxy(1,24);
						clreol();
						gotoxy(1,23);
						clreol();
						cprintf("\rOutput file name (<CR> cancels): ");
						buffer[0] = 13;
						ofile_name = cgets(buffer);
						if (buffer[1] == 0) cancel = TRUE;
                        if (!cancel)
						{
							make_filename(ofile_name);
                            out_file = fopen(ofile_name,"r");
                            if (out_file == NULL)
                            {
								cprintf("File %s does not exist. Create (Y/N) ? ",ofile_name);
								clreol();
                                i = getch();
                                i = toupper(i);
                                printf("%c",i);
                                if (i=='Y')
                                {
                                    fclose(out_file);
                                    out_file = fopen(ofile_name,"w");
                                    file_flag = TRUE;
                                }
                            }
                            else
                            {
								cprintf("File %s already exists. [O]verwrite/[A]ppend/[C]ancel: ",ofile_name);
                                i = getch();
                                i = toupper(i);
                                printf("%c",i);
                                switch (i)
                                {
                                    case ('O'):
                                    {
                                        fclose(out_file);
                                        out_file = fopen(ofile_name,"w");
                                        file_flag = TRUE;
                                        break;
                                    }
                                    case ('A'):
                                    {
                                        fclose(out_file);
                                        out_file = fopen(ofile_name,"a");
                                        file_flag = TRUE;
                                        break;
                                    }
                                    case ('C'):
                                    {
                                        cancel = TRUE;
                                        break;
                                    }
                                    default:
                                    {
                                        break;
                                    }
                                }
                            }
                        }
                    }
                    if ((file_flag) && (!cancel))
					{
						cprintf("\rPlease indicate how words should be added to the output file.");
						clreol();
						cprintf("\r\nFile: 1:word 2:->group 3:group-> 4:group 5:->all 6:all-> 7:all  Choice: ");
						help_flag = TRUE;
						got_fcmd = FALSE;
						while (!got_fcmd)
						{
							fcmd = getch();
							if ((fcmd >= '1') && (fcmd <= '7'))
							{
								printf("%c",fcmd);
								delay (500);
								got_fcmd = TRUE;
							}
						}
                    	show_word = TRUE;
                   	 	break;
					}
					break;
				}
                case ('Q'):
				{
					gotoxy(1,24);
					clreol();
					gotoxy(1,23);
					clreol();
                    printf("Quitting...");
                    done = TRUE;
                    if (file_flag) fclose(out_file);
                    break;
                }
                case ('E'):
                {
                    fprintf(prnt_dev,"%c\r",FORMFEED);
                    break;
                }
                case ('C'):
                {
                    if (file_flag)
					{
						gotoxy(1,24);
						clreol();
						gotoxy(1,23);
						clreol();
						printf("Closing file...");
						delay(1000);
						help_flag = TRUE;
						show_word = TRUE;
                        fclose(out_file);
                        file_flag = FALSE;
                    }
                    break;
				}
				case ('\r'):
                case ('R'):
                {
                    /* random - wait for keypress */
					rand_flag = FALSE;
                    while (!rand_flag)
                    {
						pr_arr[PR_P][PR_PTR] = random(pr_arr[PR_P][PR_MAX]+1);
						pr_arr[PR_S][PR_PTR] = random(pr_arr[PR_S][PR_MAX]+1);
						display_word(prefixes[pr_arr[PR_P][PR_PTR]],suffixes[pr_arr[PR_S][PR_PTR]],pref_def[pr_arr[PR_P][PR_PTR]],suff_def[pr_arr[PR_S][PR_PTR]],'R',TRUE);
						if (sound_flag) sound(2000);
						delay(20);
						if (sound_flag) nosound();
						delay(50);
						if (kbhit())
                        {
                            ch = getch();
							rand_flag = TRUE;
                        }
                    }
                    show_word = TRUE;
                    break;
                }
                case ('P'):
                {
					gotoxy(1,24);
					clreol();
					gotoxy(1,23);
					clreol();
					cprintf("\rPlease indicate how words should be printed.");
					cprintf("\r\nPrint: 1:word 2:->group 3:group-> 4:group 5:->all 6:all-> 7:all  Choice: ");
					got_pcmd = FALSE;
					while (!got_pcmd)
					{
						pcmd = getch();
						if ((pcmd >= '1') && (pcmd <= '7'))
						{
							printf("%c",pcmd);
							delay(500);
							got_pcmd = TRUE;
						}
					}
					show_word = TRUE;
					help_flag = TRUE;
                    break;
                }
                case ('\0'):
                {
                    command = getch();
                    switch (command)
                    {
                        case ('\073'):  /* F1 */
                        {
                            show_help();
                            help_flag = TRUE;
                            show_word = TRUE;
                            break;
                        }
                        case ('\074'):  /* F2 */
						{
							gotoxy(1,24);
							clreol();
							gotoxy(1,23);
							clreol();
							printf("Printing...");
		                    switch (pcmd)
                    		{
								case ('1'):
								/* word */
								{
									fprintf(prnt_dev,"          %s%s   %s + %s\n",prefixes[pr_arr[PR_P][PR_PTR]],suffixes[pr_arr[PR_S][PR_PTR]],pref_def[pr_arr[PR_P][PR_PTR]],suff_def[pr_arr[PR_S][PR_PTR]]);
		                            break;
		                        }
								case ('4'):
								/* group */
								{
		                            for (i=0; i<=pr_arr[non_prior][PR_MAX]; i++)
		                            {
		                                p_indx = ((priority==PR_P)?pr_arr[PR_P][PR_PTR]:i);
		                                s_indx = ((priority==PR_S)?pr_arr[PR_S][PR_PTR]:i);
		                                fprintf(prnt_dev,"          %s%s\n",prefixes[p_indx],suffixes[s_indx]);
		                            }
		                            break;
		                        }
								case ('2'):
								/* group from start to current */
								{
		                            for (i=0; i<=pr_arr[non_prior][PR_PTR]; i++)
		                            {
		                                p_indx = ((priority==PR_P)?pr_arr[PR_P][PR_PTR]:i);
		                                s_indx = ((priority==PR_S)?pr_arr[PR_S][PR_PTR]:i);
		                                fprintf(prnt_dev,"          %s%s\n",prefixes[p_indx],suffixes[s_indx]);
		                            }

		                            break;
		                        }
								case ('3'):
								/* group from current to end */
								{
		                            for (i=pr_arr[non_prior][PR_PTR]; i<=pr_arr[non_prior][PR_MAX]; i++)
		                            {
		                                p_indx = ((priority==PR_P)?pr_arr[PR_P][PR_PTR]:i);
		                                s_indx = ((priority==PR_S)?pr_arr[PR_S][PR_PTR]:i);
		                                fprintf(prnt_dev,"          %s%s\n",prefixes[p_indx],suffixes[s_indx]);
		                            }
		                            break;
		                        }
								case ('7'):
								/* all */
								{
		                            for (j=0; j<=pr_arr[priority][PR_MAX]; j++)
		                            {
		                                for (i=0; i<=pr_arr[non_prior][PR_MAX]; i++)
		                                {
		                                    p_indx = ((priority==PR_P)?j:i);
		                                    s_indx = ((priority==PR_S)?j:i);
		                                    fprintf(prnt_dev,"          %s%s\n",prefixes[p_indx],suffixes[s_indx]);
		                                }
		                            }
		                            break;
		                        }
								case ('5'):
								/* all groups from start to current group */
								{
		                            for (j=0; j<=pr_arr[priority][PR_PTR]; j++)
		                            {
		                                for (i=0; i<=pr_arr[non_prior][PR_MAX]; i++)
		                                {
		                                    p_indx = ((priority==PR_P)?j:i);
		                                    s_indx = ((priority==PR_S)?j:i);
		                                    fprintf(prnt_dev,"          %s%s\n",prefixes[p_indx],suffixes[s_indx]);
		                                }
		                            }
		                            break;
		                        }
								case ('6'):
								/* all from current group to last group */
								{
		                            for (j=pr_arr[priority][PR_PTR]; j<=pr_arr[priority][PR_MAX]; j++)
		                            {
		                                for (i=0; i<=pr_arr[non_prior][PR_MAX]; i++)
		                                {
		                                    p_indx = ((priority==PR_P)?j:i);
		                                    s_indx = ((priority==PR_S)?j:i);
		                                    fprintf(prnt_dev,"          %s%s\n",prefixes[p_indx],suffixes[s_indx]);
		                                }
		                            }
		                            break;
		                        }
		                        default:
		                        {
		                            /* invalid printer command - ignore */
		                            break;
		                        }
		                    }
							delay(500);
							show_word = TRUE;
                            break;
                        }
                        case ('\075'):  /* F3 */
                        {
                            if (!file_flag)
							{
								gotoxy(1,24);
								clreol();
								gotoxy(1,23);
								clreol();
                                printf("Use 'F' option to name output file first. Press <Enter>: ");
                                getch();
                                show_word = TRUE;
                            }
                            else
							{
								gotoxy(1,24);
								clreol();
								gotoxy(1,23);
								clreol();
								printf("Writing to file...");
								switch (fcmd)
								{
									case ('1'):
									{
										fprintf(out_file,"%s%s   %s + %s\n",prefixes[pr_arr[PR_P][PR_PTR]],suffixes[pr_arr[PR_S][PR_PTR]],pref_def[pr_arr[PR_P][PR_PTR]],suff_def[pr_arr[PR_S][PR_PTR]]);
										break;
									}
									case ('4'):
									{
										for (i=0; i<=pr_arr[non_prior][PR_MAX]; i++)
										{
											p_indx = ((priority==PR_P)?pr_arr[PR_P][PR_PTR]:i);
											s_indx = ((priority==PR_S)?pr_arr[PR_S][PR_PTR]:i);
											fprintf(out_file,"%s%s\n",prefixes[p_indx],suffixes[s_indx]);
										}
										break;
									}
									case ('2'):
									{
										for (i=0; i<=pr_arr[non_prior][PR_PTR]; i++)
										{
											p_indx = ((priority==PR_P)?pr_arr[PR_P][PR_PTR]:i);
											s_indx = ((priority==PR_S)?pr_arr[PR_S][PR_PTR]:i);
											fprintf(out_file,"%s%s\n",prefixes[p_indx],suffixes[s_indx]);
										}
										break;
									}
									case ('3'):
									{
										for (i=pr_arr[non_prior][PR_PTR]; i<=pr_arr[non_prior][PR_MAX]; i++)
										{
											p_indx = ((priority==PR_P)?pr_arr[PR_P][PR_PTR]:i);
											s_indx = ((priority==PR_S)?pr_arr[PR_S][PR_PTR]:i);
											fprintf(out_file,"%s%s\n",prefixes[p_indx],suffixes[s_indx]);
										}
										break;
									}
									case ('7'):
									{
										for (j=0; j<=pr_arr[priority][PR_MAX]; j++)
										{
											for (i=0; i<=pr_arr[non_prior][PR_MAX]; i++)
											{
												p_indx = ((priority==PR_P)?j:i);
												s_indx = ((priority==PR_S)?j:i);
												fprintf(out_file,"%s%s\n",prefixes[p_indx],suffixes[s_indx]);
											}
										}
										break;
									}
									case ('5'):
									{
										for (j=0; j<=pr_arr[priority][PR_PTR]; j++)
										{
											for (i=0; i<=pr_arr[non_prior][PR_MAX]; i++)
											{
												p_indx = ((priority==PR_P)?j:i);
												s_indx = ((priority==PR_S)?j:i);
												fprintf(out_file,"%s%s\n",prefixes[p_indx],suffixes[s_indx]);
											}
										}
										break;
									}
									case ('6'):
									{
										for (j=pr_arr[priority][PR_PTR]; j<=pr_arr[priority][PR_MAX]; j++)
										{
											for (i=0; i<=pr_arr[non_prior][PR_MAX]; i++)
											{
												p_indx = ((priority==PR_P)?j:i);
												s_indx = ((priority==PR_S)?j:i);
												fprintf(out_file,"%s%s\n",prefixes[p_indx],suffixes[s_indx]);
											}
										}
										break;
									}
									default:
									{
										/* invalid file command - ignore */
										break;
									}
								}
								printf("Ok.");
								clreol();
								delay(500);
								show_word = TRUE;
                            }
                            break;
                        }
                        case ('\113'):  /* left arrow */
                        {
							/* priority = PR_P; */
							priority=((priority==PR_S)?PR_P:PR_S);
							show_word = TRUE;
                            break;
                        }
                        case ('\115'):  /* right arrow */
                        {
							/* priority = PR_S; */
							priority=((priority==PR_S)?PR_P:PR_S);
							show_word = TRUE;
                            break;
                        }
                        case ('\110'):  /* up arrow */
                        {
                            pr_arr[non_prior][PR_PTR]--;
                            if (pr_arr[non_prior][PR_PTR] < 0)
                                pr_arr[non_prior][PR_PTR] = pr_arr[non_prior][PR_MAX];
                            show_word = TRUE;
                            break;
                        }
                        case ('\120'):  /* down arrow */
                        {
                            pr_arr[non_prior][PR_PTR]++;
                            if (pr_arr[non_prior][PR_PTR] > pr_arr[non_prior][PR_MAX])
                                pr_arr[non_prior][PR_PTR] = 0;
                            show_word = TRUE;
                            break;
                        }
                        case ('\107'):  /* home */
                        {
                            pr_arr[non_prior][PR_PTR] = 0;
                            show_word = TRUE;
                            break;
                        }
                        case ('\117'):  /* end */
                        {
                            pr_arr[non_prior][PR_PTR] = pr_arr[non_prior][PR_MAX];
                            show_word = TRUE;
                            break;
                        }
                        case ('\111'):  /* PgUp */
                        {
                            pr_arr[priority][PR_PTR]--;
                            if (pr_arr[priority][PR_PTR] < 0)
                                pr_arr[priority][PR_PTR] = pr_arr[priority][PR_MAX];
                            show_word = TRUE;
                            break;
                        }
                        case ('\121'):  /* PgDn */
                        {
                            pr_arr[priority][PR_PTR]++;
                            if (pr_arr[priority][PR_PTR] > pr_arr[priority][PR_MAX])
                                pr_arr[priority][PR_PTR] = 0;
                            show_word = TRUE;
                            break;
                        }
                        case ('\167'):  /* Ctrl-Home */
                        {
                            pr_arr[priority][PR_PTR] = 0;
                            pr_arr[non_prior][PR_PTR] = 0;
                            show_word = TRUE;
                            break;
                        }
                        case ('\165'):  /* Ctrl-End */
                        {
                            pr_arr[priority][PR_PTR] = pr_arr[priority][PR_MAX];
                            pr_arr[non_prior][PR_PTR] = pr_arr[non_prior][PR_MAX];
                            show_word = TRUE;
                            break;
                        }
                        case ('\204'):  /* Ctrl-PgUp */
                        {
                            pr_arr[priority][PR_PTR] = 0;
                            show_word = TRUE;
                            break;
                        }
                        case ('\166'):  /* Ctrl-PgDn */
                        {
                            pr_arr[priority][PR_PTR] = pr_arr[priority][PR_MAX];
                            show_word = TRUE;
                            break;
                        }
                        default:
                        {
                            /* not a valid extended command - ignore */
                            break;
                        }
                    }
                }
                default:
                {
                    /* not a valid command - ignore */
                    break;
                }
            }
        }
		cprintf("\r\n\r\nProgram terminated.\r\n");
		return(0);
	}


